@@ -18,6 +18,7 @@ from .charge_card_transactions import ChargeCardTransactions
 from .customers import Customers
 from .items import Items
 from .ap_payments import APPayments
+from .ar_invoices import ARInvoices
 from .reimbursements import Reimbursements
 from .checking_accounts import CheckingAccounts
 from .savings_accounts import SavingsAccounts
@@ -25,8 +26,12 @@ from .dimensions import Dimensions
 from .dimension_values import DimensionValues
 from .tasks import Tasks
 from .expense_payment_types import ExpensePaymentTypes
+from .location_entities import LocationEntities
+from .tax_details import TaxDetails
+from .gl_detail import GLDetail
+from .classes import Classes
 
-__all_ = [
+__all__ = [
     'ApiBase',
     'Contacts',
     'Locations',
@@ -44,11 +49,16 @@ __all_ = [
     'Customers',
     'Items',
     'APPayments',
+    'ARInvoices',
     'Reimbursements',
     'CheckingAccounts',
     'SavingsAccounts',
     'Dimensions',
     'DimensionValues',
     'Tasks',
-    'ExpensePaymentTypes'
+    'ExpensePaymentTypes',
+    'LocationEntities',
+    'TaxDetails',
+    'GLDetail',
+    'Classes'
 ]
