@@ -4,7 +4,7 @@ Sage Intacct Python SDK
 from .apis import ApiBase, Contacts, Locations, Employees, Accounts, ExpenseTypes, Attachments, ExpenseReports,\
     Vendors, Bills, Projects, Departments, ChargeCardAccounts, ChargeCardTransactions, Customers, Items,\
     APPayments, Reimbursements, CheckingAccounts, SavingsAccounts, Tasks, ExpensePaymentTypes, Dimensions,\
-    DimensionValues
+    DimensionValues, LocationEntities, ARInvoices, TaxDetails, GLDetail, Classes
 
 
 class SageIntacctSDK:
@@ -12,7 +12,8 @@ class SageIntacctSDK:
     Sage Intacct SDK
     """
 
-    def __init__(self, sender_id: str, sender_password: str, user_id: str, company_id: str, user_password: str):
+    def __init__(self, sender_id: str, sender_password: str, user_id: str,
+        company_id: str, user_password: str, entity_id: str=None):
         """
         Initialize connection to Sage Intacct
         :param sender_id: Sage Intacct sender id
@@ -20,6 +21,7 @@ class SageIntacctSDK:
         :param user_id: Sage Intacct user id
         :param company_id: Sage Intacct company id
         :param user_password: Sage Intacct user password
+        :param (optional) entity_id: Sage Intacct entity ID
         """
         # Initializing variables
         self.__sender_id = sender_id
@@ -27,6 +29,7 @@ class SageIntacctSDK:
         self.__user_id = user_id
         self.__company_id = company_id
         self.__user_password = user_password
+        self.__entity_id = entity_id
 
         self.api_base = ApiBase()
         self.contacts = Contacts()
@@ -45,6 +48,7 @@ class SageIntacctSDK:
         self.customers = Customers()
         self.items = Items()
         self.ap_payments = APPayments()
+        self.ar_invoices = ARInvoices()
         self.reimbursements = Reimbursements()
         self.checking_accounts = CheckingAccounts()
         self.savings_accounts = SavingsAccounts()
@@ -52,7 +56,10 @@ class SageIntacctSDK:
         self.dimension_values = DimensionValues()
         self.tasks = Tasks()
         self.expense_payment_types = ExpensePaymentTypes()
-
+        self.location_entities = LocationEntities()
+        self.tax_details = TaxDetails()
+        self.gl_detail = GLDetail()
+        self.classes = Classes()
         self.update_sender_id()
         self.update_sender_password()
         self.update_session_id()
@@ -78,6 +85,7 @@ class SageIntacctSDK:
         self.customers.set_sender_id(self.__sender_id)
         self.items.set_sender_id(self.__sender_id)
         self.ap_payments.set_sender_id(self.__sender_id)
+        self.ar_invoices.set_sender_id(self.__sender_id)
         self.reimbursements.set_sender_id(self.__sender_id)
         self.checking_accounts.set_sender_id(self.__sender_id)
         self.savings_accounts.set_sender_id(self.__sender_id)
@@ -85,6 +93,10 @@ class SageIntacctSDK:
         self.dimension_values.set_sender_id(self.__sender_id)
         self.tasks.set_sender_id(self.__sender_id)
         self.expense_payment_types.set_sender_id(self.__sender_id)
+        self.location_entities.set_sender_id(self.__sender_id)
+        self.tax_details.set_sender_id(self.__sender_id)
+        self.gl_detail.set_sender_id(self.__sender_id)
+        self.classes.set_sender_id(self.__sender_id)
 
     def update_sender_password(self):
         """
@@ -107,6 +119,7 @@ class SageIntacctSDK:
         self.customers.set_sender_password(self.__sender_password)
         self.items.set_sender_password(self.__sender_password)
         self.ap_payments.set_sender_password(self.__sender_password)
+        self.ar_invoices.set_sender_password(self.__sender_password)
         self.reimbursements.set_sender_password(self.__sender_password)
         self.checking_accounts.set_sender_password(self.__sender_password)
         self.savings_accounts.set_sender_password(self.__sender_password)
@@ -114,12 +127,17 @@ class SageIntacctSDK:
         self.dimension_values.set_sender_password(self.__sender_password)
         self.tasks.set_sender_password(self.__sender_password)
         self.expense_payment_types.set_sender_password(self.__sender_password)
+        self.location_entities.set_sender_password(self.__sender_password)
+        self.tax_details.set_sender_password(self.__sender_password)
+        self.gl_detail.set_sender_password(self.__sender_password)
+        self.classes.set_sender_password(self.__sender_password)
 
     def update_session_id(self):
         """
         Update the session id and change it in all API objects.
         """
-        self.__session_id = self.api_base.get_session_id(self.__user_id, self.__company_id, self.__user_password)
+        self.__session_id = self.api_base.get_session_id(
+        self.__user_id, self.__company_id, self.__user_password, self.__entity_id)
         self.api_base.set_session_id(self.__session_id)
         self.contacts.set_session_id(self.__session_id)
         self.locations.set_session_id(self.__session_id)
@@ -137,6 +155,7 @@ class SageIntacctSDK:
         self.customers.set_session_id(self.__session_id)
         self.items.set_session_id(self.__session_id)
         self.ap_payments.set_session_id(self.__session_id)
+        self.ar_invoices.set_session_id(self.__session_id)
         self.reimbursements.set_session_id(self.__session_id)
         self.checking_accounts.set_session_id(self.__session_id)
         self.savings_accounts.set_session_id(self.__session_id)
@@ -144,3 +163,7 @@ class SageIntacctSDK:
         self.dimension_values.set_session_id(self.__session_id)
         self.tasks.set_session_id(self.__session_id)
         self.expense_payment_types.set_session_id(self.__session_id)
+        self.location_entities.set_session_id(self.__session_id)
+        self.tax_details.set_session_id(self.__session_id)
+        self.gl_detail.set_session_id(self.__session_id)
+        self.classes.set_session_id(self.__session_id)
